# SPDX-FileCopyrightText: Copyright (c) 2021 Tim C
#
# SPDX-License-Identifier: MIT
"""
`displayio_listselect`
================================================================================

ListSelect widget for CircuitPython DisplayIO.
  Display a list of strings with a selection indicator allow
  the user to move selection up and down.


* Author(s): Tim C

Implementation Notes
--------------------

**Hardware:**

Any CircuitPython device with displayio support.

**Software and Dependencies:**

* Adafruit CircuitPython firmware for the supported boards:
  https://circuitpython.org/downloads

"""

import terminalio
from adafruit_display_text.bitmap_label import Label
from displayio import Group


__version__ = "1.0.0"
__repo__ = "https://github.com/foamyguy/CircuitPython_DisplayIO_ListSelect.git"


class ListSelect(Group):
    """
    ListSelect widget for CircuitPython DisplayIO.
    Display a list of strings with a selection indicator allow
    the user to move selection up and down.
    """

    # pylint:disable=too-many-arguments
    def __init__(
        self,
        items=None,
        font=terminalio.FONT,
        x=0,
        y=0,
        color=0xFFFFFF,
        background_color=0x000000,
        selected_index=0,
        visible_items_count=None,
        cursor_char=">",
        **kwargs
    ):
        super().__init__(x=x, y=y, scale=1)
        self._label = Label(
            font, text="", color=color, background_color=background_color, **kwargs
        )
        self._label.anchor_point = (0, 0)
        self._label.anchored_position = (0, 0)

        if visible_items_count == 0:
            raise ValueError(
                "Must have at least 1 visible item. Or None for unrestricted"
            )
        self.visible_items_count = visible_items_count
        self.visible_index = 0

        self.items = items
        self._selected_index = selected_index
        self.cursor_char = cursor_char

        self.append(self._label)
        self._refresh_label()

    def _refresh_label(self):
        """Called any time that we need to update the displayed label."""
        _full_str = ""
        if self.visible_items_count:
            loop_from = min(
                self.visible_index, len(self.items) - self.visible_items_count
            )
            loop_to = min(
                len(self.items), self.visible_index + self.visible_items_count
            )
        else:
            loop_from = 0
            loop_to = len(self.items)
        # print(f"showing indexes {self.visible_index} - {loop_to}")
        for i in range(loop_from, loop_to):
            item = self.items[i]

            if i == self.selected_index:
                _full_str += self.cursor_char
            else:
                _full_str += " "
            _full_str += item

            if i != len(self.items) - 1:
                _full_str += "\n"

        self._label.text = _full_str

    def move_selection_down(self):
        """
        Move the selection indicator down 1 space
        :return: None
        """
        if self.selected_index + 1 < len(self.items):
            self.selected_index += 1
            self._refresh_label()

    def move_selection_up(self):
        """
        Move the selection indicator up 1 space
        :return: None
        """
        if self.selected_index - 1 >= 0:
            self.selected_index -= 1
            self._refresh_label()

    @property
    def width(self):
        """The widget width, in pixels. (getter only)
        :return: int
        """
        return self._label.bounding_box[2]

    @property
    def height(self):
        """The widget height, in pixels. (getter only)
        :return: int
        """
        return self._label.bounding_box[3]

    @property
    def bounding_box(self):
        """
        The bounding box of the label that the
        list is shown within.
        :return: Tuple
        """
        return self._label.bounding_box

    def resize(self, new_width, new_height):
        """
        Not Implemented
        """
        raise NotImplementedError(
            "Label does not support arbitrary sizing, so neither does ListSelect."
        )

    @property
    def anchor_point(self):
        """
        The anchor point of the label that the list is shown within
        :return: Tuple
        """
        return self._label.anchor_point

    @anchor_point.setter
    def anchor_point(self, new_anchor_point):
        """
        The anchor point of the label that the list is shown within
        :param new_anchor_point:
        :return: None
        """
        self._label.anchor_point = new_anchor_point

    @property
    def anchored_position(self):
        """
        The anchored position of the label that the list is shown
        within
        :return: Tuple
        """
        return self._label.anchored_position

    @anchored_position.setter
    def anchored_position(self, new_anchored_position):
        """
        The anchored position of the label that the list is shown
        within
        :param new_anchored_position:
        :return: None
        """
        self._label.anchored_position = new_anchored_position

    @property
    def selected_index(self):
        """
        The currently selected index.
        :return: int
        """
        return self._selected_index

    @selected_index.setter
    def selected_index(self, new_index):
        """
        The currently selected index
        :param new_index:
        :return: None
        """
        self._selected_index = new_index
        if self.visible_items_count:
            self.visible_index = new_index
        self._refresh_label()

    @property
    def selected_item(self):
        """
        The string representing the currently selected item.
        :return: String
        """
        return self.items[self._selected_index]



# SPDX-FileCopyrightText: 2020 Tim C
#
# SPDX-License-Identifier: MIT
"""
FunHouse example for ListSelect
Make a ListSelect and use up, down, and select buttons to interact with it.
"""
import time
import board
import displayio
from adafruit_funhouse.peripherals import Peripherals
from displayio_listselect import ListSelect


PREV_UP_STATE = False
PREV_DOWN_STATE = False
PREV_SELECT_STATE = False

display = board.DISPLAY
# Make the display context
main_group = displayio.Group()
display.show(main_group)

fun_house = Peripherals()
print(dir)

items = ["First", "Second", "Third", "Fourth"]

list_select = ListSelect(scale=2, items=items)

main_group.append(list_select)

list_select.anchor_point = (0.5, 0.5)
list_select.anchored_position = (display.width // 2, display.height // 2)


while True:

    if fun_house.button_up and not PREV_UP_STATE:
        list_select.move_selection_up()
    if fun_house.button_down and not PREV_DOWN_STATE:
        list_select.move_selection_down()
    if fun_house.button_sel and not PREV_SELECT_STATE:
        print(list_select.selected_item)

    PREV_UP_STATE = fun_house.button_up
    PREV_DOWN_STATE = fun_house.button_down
    PREV_SELECT_STATE = fun_house.button_sel

    time.sleep(0.05)



# SPDX-FileCopyrightText: 2020 Tim C
#
# SPDX-License-Identifier: MIT
"""
FunHouse example for ListSelect
Make a ListSelect and use up, down, and select buttons to interact with it.
"""
import time
import board
import displayio
import analogio
from displayio_listselect import ListSelect

# state variables for the joystick states we are interested in
STATE_JOYSTICK_DOWN = 0
STATE_JOYSTICK_UP = 1
STATE_JOYSTICK_NEUTRAL = 2

# current and previous iteration state variables
CUR_STATE = STATE_JOYSTICK_NEUTRAL
PREV_STATE = STATE_JOYSTICK_NEUTRAL

# setup analog in for the joystick Y (up/down) direction.
# This pin is for the PyGamer, change to the GPIO connected to your
# Joystick Y axis if not using PyGamer
joystick_y = analogio.AnalogIn(board.JOYSTICK_Y)

# use built-in display
display = board.DISPLAY

# Make the display context
main_group = displayio.Group()
# show our group
display.show(main_group)

# the items in the selectable list
items = ["First", "Second", "Third", "Fourth"]

# Create ListSelect object
list_select = ListSelect(scale=2, items=items)

# add it to the group so it will show on the display
main_group.append(list_select)

# center list select in the middle of the display
list_select.anchor_point = (0.5, 0.5)
list_select.anchored_position = (display.width // 2, display.height // 2)

# initial analog value of joystick
cur_value = joystick_y.value

while True:
    # update the analog value of joystick
    cur_value = joystick_y.value

    if cur_value < 15000:  # pygamer joystick up will have a value below 15,000
        CUR_STATE = STATE_JOYSTICK_UP
    elif cur_value > 50000:  # pygamer joystick down will have a value above 50,000
        CUR_STATE = STATE_JOYSTICK_DOWN
    else:  # pygamer joystick neutral will have a value around 32,000
        CUR_STATE = STATE_JOYSTICK_NEUTRAL

    # if we were in neutral, and now joystick is pushed up
    if PREV_STATE == STATE_JOYSTICK_NEUTRAL and CUR_STATE == STATE_JOYSTICK_UP:
        list_select.move_selection_up()

    # if we were in neutral, and now joystick is pushed down
    if PREV_STATE == STATE_JOYSTICK_NEUTRAL and CUR_STATE == STATE_JOYSTICK_DOWN:
        list_select.move_selection_down()

    # update previous state for next time
    PREV_STATE = CUR_STATE

    # small sleep
    time.sleep(0.05)



# SPDX-FileCopyrightText: Copyright (c) 2021 Tim C
#
# SPDX-License-Identifier: MIT
"""
Create a list with a few options and move the selection around programmatically.

Runs on a PC / RasPi in CPython using Blinka_DisplayIO and PyGameDisplay
"""
import time

import displayio
from blinka_displayio_pygamedisplay import PyGameDisplay
from displayio_listselect import ListSelect

# Make the display context. Change size if you want
display = PyGameDisplay(width=320, height=240)

# Make the display context
main_group = displayio.Group()
display.show(main_group)

items = ["First", "Second", "Third", "Fourth"]

list_select = ListSelect(scale=2, items=items)

main_group.append(list_select)

list_select.anchor_point = (0.5, 0.5)
list_select.anchored_position = (display.width // 2, display.height // 2)

print(list_select.anchor_point)
print(list_select.anchored_position)
print(list_select.x)
print(list_select.bounding_box)
# print(list_select._bounding_box)

for i in range(3):
    list_select.move_selection_down()
    time.sleep(1)

for i in range(3):
    list_select.move_selection_up()
    time.sleep(1)

list_select.selected_index = 3
while display.running:
    pass
